package speiger.src.coreengine.rendering.input.divices;

import org.lwjgl.glfw.GLFW;

import speiger.src.collections.ints.sets.IntOpenHashSet;
import speiger.src.collections.ints.sets.IntSet;
import speiger.src.coreengine.math.vector.ints.Vec2i;
import speiger.src.coreengine.rendering.input.divices.Mouse.MouseData;
import speiger.src.coreengine.rendering.input.divices.Mouse.MouseTask;
import speiger.src.coreengine.rendering.input.window.Window;

public class Mouse extends AbstractDevice<MouseData, MouseTask> {
	
	@Override
	protected MouseData createData(long windowId) { return new MouseData(); }
	protected void process(MouseTask task) { task.process(this); }
	
	@Override
	protected void registerCallbacks(Window window) {
		super.registerCallbacks(window);
		window.addCallback(this::move, GLFW::glfwSetCursorPosCallback);
		window.addCallback(this::click, GLFW::glfwSetMouseButtonCallback);
		window.addCallback(this::enter, GLFW::glfwSetCursorEnterCallback);
		window.addCallback(this::scroll, GLFW::glfwSetScrollCallback);
	}
	
	private void move(long window, double x, double y) {
		push(window, new Move(window, x, y));
	}
	
	private void click(long window, int button, int action, int mods) {
		push(window, new Click(window, button, action, mods));
	}
	
	private void enter(long window, boolean enter) {
		push(window, new Enter(window, enter));
	}
	
	private void scroll(long window, double xoffset, double yoffset) {
		push(window, new Scroll(window, xoffset, yoffset));
	}
	
	public static class MouseData {
		IntSet buttons = new IntOpenHashSet();
		Vec2i position = Vec2i.mutable();
		Vec2i movement = Vec2i.mutable();
		Vec2i scroll = Vec2i.mutable();
		boolean active = true;
	}
	
	public interface MouseTask {
		public void process(Mouse mouse);
	}
	
	private record Move(long window, double x, double y) implements MouseTask {
		@Override
		public void process(Mouse mouse) {
			MouseData data = mouse.getData(window);
			int xOff = (int)(x - data.position.x());
			int yOff = (int)(y - data.position.y());
			data.position.set((int)x, (int)y);
			//TODO Post event!
		}
		
	}
	private record Click(long window, int button, int action, int mods) implements MouseTask {

		@Override
		public void process(Mouse mouse) {
			MouseData data = mouse.getData(window);
			if(action == GLFW.GLFW_PRESS) data.buttons.add(action);
			else if(action == GLFW.GLFW_RELEASE) data.buttons.remove(action);
			//TODO post event
		}
		
	}
	
	private record Scroll(long window, double xoffset, double yoffset) implements MouseTask {

		@Override
		public void process(Mouse mouse) {
			//TODO post event
		}
		
	}
	private record Enter(long window, boolean enter) implements MouseTask {

		@Override
		public void process(Mouse mouse) {
			MouseData data = mouse.getData(window);
			data.active = enter;
			if(!enter) {
				//TODO decide if this should also push phantom events clearing the pressed keys?
				data.buttons.clear();
				data.movement.negate();
				data.position.negate();
				data.scroll.negate();
			}
		}
		
	}
}